import copy, datetime, itertools
from .misc import (cell, blank)

######################################
#                           Trial
######################################

def trial(s, p, maxstep, verbose, b, pair):
    if len(pair) == 0: return (s, p, 0, '', False, False)
    for x in pair:
        i = x[0]
        for j in x[1:]:
            s2 = copy.deepcopy(s)
            p2 = copy.deepcopy(p)
            s2[i] = j
            p2[i] = [10]
            found = True
            minblank = blank(s2) - maxstep
            if minblank < 0: minblank = 0
            message2 = ''
            line = 1
            while found and blank(s2) > minblank:
                s2, p2, message3, found, err = scan(s2,p2,4,b)
                message2 += '('+str(line)+') '+message3+'\n'
                line += 1
            if err:
                message = ''
                if verbose > 1:
                    if verbose == 2:
                        message = 'Determine '+cell(i)+' by trial.'
                    else:
                        message = 'Trial. If '+cell(i)+' is '+str(j)+',\n' + message2.replace('This sudoku has no solution because', 'Contradiction:')
                p[i][j-1] = 0
                s[i] = p[i].index(1)+1
                step = message2.count('=')
                if verbose > 2:
                    message += 'Therefore '+cell(i)+' is '+str(s[i])+'.'
                return (s, p, step, message, True, False)
    return (s, p, 0, '', False, False)
    
def scan(s,p,verbose,b):
    from .calc import (naksing, hidsing)
    from .misc import check
    s, p, message, found, err = naksing(s, p, b, verbose) # Naked single
    if found or err: return (s, p, message, found, err)
    message, err = check(s)
    if err: return (s, p, message, False, True)
    s, p, message, found, err = hidsing(s, p, verbose) # Hidden single
    return (s, p, message, found, err)

###########################################################
#                                                   Full search
###########################################################

def search(s, p, verbose, depth, maxdepth, endtime, b, pb):
    from .calc import (solveone)

    if datetime.datetime.now() > endtime:
        message = ''
        if verbose > 0: message = 'Search time limit'
        return (s, p, message, 9+depth, depth, False, False)
    
    message = ''
    smallest = 10
    pair = []
    pairnum = [[],[],[],[],[],[],[],[],[]]
    for j in range(81):
        if s[j] == 0:
            if sum(p[j]) == 2:
                pair.append(j)
                for k in range(9):
                    if p[j][k] == 1: pairnum[k].append ([j])
            if sum(p[j]) < smallest: i = j; smallest = sum(p[j])
    if len(pair) > 1:
        m = 0
        for j in range(9):
            if len(pairnum[j]) > m: i = int(pairnum[j][0][0])
    
    solution = []
    removed = []

    for n in range(9):
        if p[i][n] == 0: continue
        s2 = copy.deepcopy(s)
        p2 = copy.deepcopy(p)
        s2[i] = n+1
        while (True):
             s2, p2, message, logic, depth2, found, err = solveone(s2, p2, 0, depth, maxdepth, endtime, b, pb)
             if found:
                 if err: # Multiple solutions
                      if depth2 > depth: depth = depth2
                      return (s2, p2, message, logic, depth, True, True)
                 else:
                      if blank(s2) == 0:
                          solution.append ([n, s2])
                          break
             else:
                  if err: # No solution
                       if depth2 > depth: depth = depth2
                       removed.append (n)
                       break
                  else:
                       break
        if len(solution) > 1: break       

    if depth == 1:
        logic = 'Search'
    else:
        logic = 'Deep search'
         
    if len(solution) > 1:
        if verbose > 1:
            message = 'Search with depth '+str(depth) +' shows that this sudoku has multiple solutions.'
        return ([solution[0][1], solution[1][1]], p, message, logic, depth, True, True)
    if len(removed) > 0:
        for n in removed:
            p[i][n] = 0
        if sum(p[i]) == 0:
            if verbose > 1: message = 'Search with depth '+str(depth)+' shows that '+cell(i)+' has no candidate.'
            return (s, p, message, logic, depth, False, True)
        if sum(p[i]) == 1:
            s[i] = p[i].index(1)+1
            if verbose > 1: message = 'Search with depth '+str(depth)+' shows that '+cell(i)+' should be ' + str(s[i])
            return (s, p, message, logic, depth, True, False)
        if verbose > 1: message = 'Search with depth '+str(depth)+' shows that '+cell(i)+' is not ' + str(removed)
        return (s, p, message, logic, depth, True, False)
    else:
        if verbose > 1: message = 'Search with depth '+str(depth)+' on '+ cell(i) + ' did not give any clue.'
        return (s, p, message, logic, depth, False, False)



# -*- coding: utf-8 -*-
import copy, warnings

def conv(problem):

    s = []
    if problem.find(',') == -1:
        for i in problem:
            if i.isdigit():
                 s = s + [int(i)]
            else:
                 s = s + [0]
        if len(s) != 81:
            return ('Error in input.')
    else:            
        for line in problem.split(","):
            if (len(line)) == 9:
                for i in line:
                    if i.isdigit():
                        s = s + [int(i)]
                    else:
                        s = s + [0]
            else:
                return ('Error in input: '+line,True)

    if len(s) == 81:
        return (s, False)
    else:
        return ('Error in input',True)

def check(s):
    b = box()
    for i in range(81):
        if s[i] > 0:
            for j in b[i]:
                if s[i] == s[j]:
                    return ('Both '+ cell(i) + ' and ' + cell(j) + ' have the same value of ' + str(s[i]) + '.', True)
    return ('', False)        

def box():
    box = []
    for i in range(9):
        for j in range(9):
            b = [0] * 81
            for i2 in range(9):
                b[i2*9+j] = 1
            for j2 in range(9):
                b[i*9+j2] = 1
            begin = (i//3)*27 + (j//3)*3
            for i2 in [0,1,2]:
                for j2 in [0,1,2]:
                    b[begin + i2*9 + j2] = 1
            b[i*9+j] = 0
            bb = []
            for i2 in range(81):
                if b[i2] == 1: bb = bb + [i2]
            box = box + [bb]
    return (box)
    
def pbox():
    pbox = {}
    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        left = [0, 0, 0, 27, 27, 27, 54, 54, 54][b]
        top = [0, 3, 6, 0, 3, 6, 0, 3, 6][b]
        box = set([])
        for i in (0,1,2,9,10,11,18,19,20): box = box | set([begin+i])
        for j in range(3):
              line = set([])
              for i in range(9): line = line | set ([left + j*9 + i])
              pbox [(b,j)] = line - box
        for j in range(3,6):
              line = set([])
              for i in range(9): line = line | set ([top + j-3 + i*9])
              pbox [(b,j)] = line - box
    return (pbox)

def boxlist(s):
    boxlist = []
    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        bo=[]
        for i in (0,1,2,9,10,11,18,19,20):
            if s[begin+i] == 0: bo.append (begin+i)
        boxlist.append (bo)
    for i in range(9):
        line = []
        for j in range(9):
            if s[i*9+j] == 0: line.append (i*9+j)
        boxlist.append(line)
    for j in range(9):
        line = []
        for i in range(9):
            if s[i*9+j] == 0: line.append (i*9+j)
        boxlist.append(line)
        
    return boxlist

def combmir(p, boxl):
    comb = []
    mirror = []
    for bo in boxl:
        co = []
        mi = []
        for i in bo:
              c = ()
              for j in range(9):
                   if p[i][j] == 1: c = c + (j+1,)
              co.append (c)
        comb.append (co)
        n = []
        for i in co:
            for j in i:
                if not j in n: n.append (j)
        n = sorted(n)
        for i in n:
            m = ()
            for j in range(len(co)):
                if i in co[j]: m = m + (j,)
            mi.append(m)
        mirror.append ([n,mi])
    return (comb, mirror)

def pairs(s, p):
    pair = []
    paircomb = []
    for i in range(81):
        if s[i] == 0:
            if sum(p[i]) == 2:
                m = p[i].index(1)+1
                n = p[i][m:].index(1)+m+1
                pair.append([i, m, n])
                paircomb.append ([i,m])
                paircomb.append ([i,n])
    return (pair, paircomb)

def cell(i):
    return ('R'+str(i // 9+1) + 'C'+str(i % 9+1))

def blank(s):
    blank = 0
    for i in range(81):
        if s[i] == 0:
            blank = blank + 1
    return (blank)

def lev(i):
    return (['undefined', 'trivial', 'very easy', 'easy', 'normal', 'hard', 'very hard', 'evil', 'extreme', 'ultimate'][i])

def duplicate(s1,s2):
    s=[]
    for i in range(81):
        if s1[i] == s2[i]:
            s.append (s1[i])
        else:
            s.append (0)
    return (s)

def status(level, solved, err):
    if solved:
        if err:
            status = 'multiple solution'
        else:
            status = 'level '+str(level)
    else:
        if err:
            status = 'no solution'
        else:
            status = 'give up'
    return status
    
def current(s, move):
    pos = 0
    for m in move:
        m = int(m)
        row = m // 100
        column = (int(m) - row * 100) // 10
        n = m % 10
        i = (row-1) * 9 + column-1
        if s[i] > 0:
            message = 'Cell '+cell(i)+' is already filled with '+str(s[i])
            if pos == 0:
                return (s, [], message, True)
            else:
                return (s, move[:pos], message, True)
        s2 = copy.copy(s)
        s2[i] = n
        message, err = check(s2)
        if err:
            if pos == 0:
                return (s, [], message, True)
            else:
                return (s, move[:pos], message, True)
        message = 'Moved: '+cell(i)+' = '+str(n)
        s = s2
        pos += 1
    return s, move, message, False

def openappend(file):
    try:
        output = open(file, 'a')
    except:
        try:
            output = open(file, 'w')
        except:
            return 'error'
    return output



# -*- coding: utf-8 -*-
import sys, copy, datetime, random
from statistics import mean, stdev
from .calc import (solve, possible)
from .misc import (conv, check, box, blank, lev, status, current, openappend)
from .output import (output, short)

def analyze(file, level, verbose):
    if file == '': return
    start = datetime.datetime.now()
    input = open(file, 'r')
    i = []
    l = []
    n = 0
    for line in input:
        n += 1
        data = line.strip().split(' ')
        if int(data[0]) == level:
            s = conv(data[1])[0]
            s2 = copy.copy (s)
            if verbose > 0:
                print ('Solving problem No.{0}'.format(len(i)+1))
            s2, message, level2, solved, err = solve(s2, verbose, 20, 10)
            if not solved and err:
                print ('\nNo solution {0}'.format(data[1]))
            if solved and err:
                print ('\nMultiple solutions {0}'.format(data[1]))
            if not solved and not err:
                print ('\nGive up {0}'.format(data[1]))
            i.append (blank(s))
            l.append (level2)
            if verbose == 0:
                if len(i) % 10 == 0:
                    print ('.', end='', flush=True)
                if len(i) % 200 == 0:
                    dt = datetime.datetime.now() - start
                    t = dt.seconds + float(dt.microseconds)/1000000
                    print (' {0} problems solved in {1:.1f} seconds (mean: {2:.3f} sec).'.format(len(i), t, t/len(i)))

    dt = datetime.datetime.now() - start
    t = dt.seconds + float(dt.microseconds)/1000000
    print ('\n{0} problems solved in {1:.1f} seconds (mean: {2:.3f} sec).'.format(len(i), t, t/(len(i))))
    for j in range(11):
        n = l.count(j)
        if n > 0:
            print ('Level {0}: {1}'.format(j, n))            
    print ('Numbers: mean {0:.1f} std {1:.1f} min {2}'.format(81-mean(i), stdev(i), 81-max(i)))            
    input.close
    return

def reanalyze(file, file2):
    if file == '': return
    start = datetime.datetime.now()
    input = open(file, 'r')
    output = open(file2, 'w')
    i = 0
    for line in input:
        data = line.strip().split(' ')
        s = conv(data[1])[0]
        s2 = copy.copy (s)
        s2, message, level, solved, err = solve(s2, 0, 20, 10)
        if not solved and err:
            print ('\nNo solution {0}'.format(data[1])); continue
        if solved and err:
            print ('\nMultiple solutions {0}'.format(data[1])); continue
        if not solved and not err:
            print ('\nGive up {0}'.format(data[1])); continue
        output.write (str(level)+' '+data[1]+'\n')
        dt = datetime.datetime.now() - start
        t = dt.seconds + float(dt.microseconds)/1000000
        i += 1        
        if i % 10 == 0:
            print ('.', end='', flush=True)
        if i % 200 == 0:
            dt = datetime.datetime.now() - start
            t = dt.seconds + float(dt.microseconds)/1000000
            print (' {0} problems reanalyzed in {1:.1f} seconds (mean: {2:.3f} sec).'.format(i, t, t/i))
    input.close
    output.close
    print ('')
    return

def append_database(file, giveup, n):

    maxtime = 3
    maxdepth = 999 # Creating mode

    if file == '': return
    output = openappend(file)
    if output == 'error':
        print ('Unable to write a file:', file)
        return
    start = datetime.datetime.now()
    for i in range(n):
        s, level = create(maxdepth,  maxtime)
        sudoku = short(s)
        print ('.', end='', flush=True)
        if level == 0:
            give = openappend(giveup)
            if output == 'error':
                print ('Unable to write a file:', giveup)
                return
            give.write ('g '+str(maxtime)+' '+sudoku+'\n')
            give.close
        else:
            output.write (str(level)+' '+sudoku+'\n')
        if (i+1) % 10 == 0 and i+1 < n:
            dt = datetime.datetime.now() - start
            t = dt.seconds + float(dt.microseconds)/1000000
            print (' {0} problems created in {1:.1f} seconds (mean: {2:.3f} sec).'.format(i+1, t, t/(i+1)))
            output.close
            output = open(file, 'a')
    output.close
    dt = datetime.datetime.now() - start
    t = dt.seconds + float(dt.microseconds)/1000000
    print ('\nFinished. {0} problems created in {1:.1f} seconds (mean: {2:.3f} sec).'.format(n, t, t/n))
    show_status(file)

    return

def show_status(file):
    if file == '': return
    input = open(file, 'r')
    l = [0] * 10
    for line in input:
        data = line.strip().split(' ')
        l[int(data[0])] += 1
    su = sum(l)
    if su == 0:
        print ('No problem in the book.')
        print ('Create problems with c command.')
        return
    for i in range(1,10):
        print ('Level {0} {1:<10}: {2:>6} ({3:5.2f} %)'.format(i,lev(i),l[i],l[i]/su*100))
    print ('Total numbers     : {0:>6}'.format(su))
    input.close
    return

def create(maxdepth, maxtime):

    min = 17 # proven minimum numbers
    
    order = []
    for i in range(81):
        order.append (i)
    err = True
    while err:
        random.shuffle(order)
        n = []; v = 0
        while (v < 8):
            for i in range(min):
                 n.append(random.randint(1,9))
                 v = len(set(n))    
        s = [0] * 81
        for i in range(8):
            s[order[i]] = i+1
        for i in range(8,min):
            p = set(range(1,10))
            for j in box()[i]:
                 if s[j] in p: p.remove(s[j])
            if len(p) == 0: continue
            s[order[i]] = int(random.sample(p, 1)[0])
        message, err = check(s)
        # if err: print ('err ', end='', flush=True) ##############
        if not err:
            s2 = copy.copy (s)
            s3 = copy.copy(s)
            start = datetime.datetime.now()
            s2, message, level, solved, err = solve(s2, 0, maxdepth, maxtime)
            dt = datetime.datetime.now() - start
            # print (status(level, solved, err)+' ', end='', flush=True) #######
#            if dt.seconds > maxtime - 2:
                # print ('\n{0} seconds for {1}: {2}'.format(dt.seconds,status(level, solved, err),short(s3)))
            if not solved and err:
                return (s3, 0)
            if solved: break
            err = True

    t = dt.seconds + float(dt.microseconds)/1000000   
    while len(s2) == 2:
        
        j = []
        p = possible(s)
        maxp = 0
        for i in range(81):
            if s2[0][i] != s2[1][i]:
                  j.append([i,sum(p[i])])
                  if sum(p[i]) > maxp: maxp = sum(p[i])
        k = []
        for i in range(len(j)):
            if j[i][1] == maxp: k.append(j[i][0])
        random.shuffle(k)
        i = k[0]
        s[i] = s2[0][i]
        s = shuffle(s)
        s2 = copy.copy (s)
        start = datetime.datetime.now()
        s2, message, level, solved, err = solve(s2, 0, maxdepth, maxtime+3)
        dt = datetime.datetime.now() - start
        t = dt.seconds + float(dt.microseconds)/1000000
    
    if level < 3:
        s = make_easy(s)
        if blank(s) < 30: level = 1
    
    return (s, level)

def make_easy(s):
    s2 = copy.copy (s)
    sol, message, level, solved, err = solve(s2, 0, 1, 1)
    if not solved or err: return(s)
    if blank(s) < 25: return s
    if blank(s) < 43:
        level = 1
    else:
        level = random.randint(1,2)
    if level == 1:
        bl = random.randint(24,28)
    else:
        bl = random.randint(37,43)
    fill = blank(s) - bl
    cell = []
    for i in range(81):
        if s[i] == 0: cell.append(i)
    random.shuffle(cell)
    for i in range(fill):
        s[cell[i]] = sol[cell[i]]

    return s

def shuffle(s):
    # rotate right
    s2 = [0]*81
    for i in range(9):
        for j in range(9):
            s2[j*9 + 8-i] = s[i*9 + j]
    s = copy.copy(s2)
    # mirror 50%
    if random.randint(1,2) == 1:
        for i in range(9):
            for j in range(9):
                s2[i*9 + 8-j] = s[i*9 + j]
    s = copy.copy(s2)
    n = [1,2,3,4,5,6,7,8,9]
    random.shuffle(n)
    for i in range(81):
        if s[i] == 0:
            s2[i] = 0
        else:
            s2[i] = n[s[i]-1]
    return (s2)

def merge(file, file2):
    if file == '': return
    if file2 == '': return
    input = open(file, 'r')
    problem = []
    new = []
    for line in input:
        data = line.strip().split(' ')
        problem.append (data[1])
    input.close
    n = countproblem(file2)
    print ('Importing {0} problems.'.format(n))
    input = open(file2, 'r')
    j = 0; k = 0
    output = openappend(file)
    if output == 'error':
        print ('Unable to write a file:', file)
        return
    for line in input:
        k += 1
        data = line.strip().split(' ')
        if data[1] not in problem:
            j += 1
            output.write (line)
    input.close
    output.close
    print ('{0} / {1} problems appended.'.format(j, k))
    return

def countproblem(file):
    input = open(file, 'r')
    n = 0
    for line in input:
        data = line.strip().split(' ')
        if conv(data[1])[1] == False:
            n += 1
    input.close
    return(n)

def reanalyze_giveup(giveup, t):
    maxdepth = 999 # Creating mode
    give = givedata(giveup, t)
    for i in give[4]:
        print ('Invalid data: ',i)
    if len(give[3]) == 0:
        print ('No position to analyze.')
        return
    print ('Reanalyzing {0} giveup positions for {1} seconds.'.format(len(give[3]), t))
    while True:
        give = givedata(giveup, t)
        if len(give[3]) == 0: break
        prob = give[3][0][1]
        s, err = conv(prob)
        print ('Analyzing ... ',end='', flush=True)
        start = datetime.datetime.now()
        dt = datetime.datetime.now() - start
        s, message, level, solved, err = solve(s, 0, maxdepth, t)
        if solved:
            if err: # Discard multiple solutions
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                writegive(giveup, give)
                print ('multiple solutions. {0} left.'.format(len(give[3])))
            else:
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                give[0].append('u '+str(dt.seconds)+' '+prob)
                writegive(giveup, give)
                print ('unique solution. {0} left.'.format(len(give[3])))
        else:
            if err:
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                if dt.seconds > 3: # Discard no solution shorter than 3 seconds
                    give[1].append('n '+str(dt.seconds)+' '+prob)
                writegive(giveup, give)
                print ('no solution. {0} left.'.format(len(give[3])))
            else:
                give = givedata(giveup, t)
                give[3] = give[3][1:]
                give[2].append('g '+str(t)+' '+prob)
                writegive(giveup, give)
                print ('give up. {0} left.'.format(len(give[3])))
    return

def givedata(giveup, t):
    input = open(giveup, 'r')
    uniq = []; no = []; long = []; short = []; invalid = []
    for line in input:
        data = line.strip().split(' ')
        if data[0] == 'u':
            uniq.append(line); continue
        if data[0] == 'n':
            no.append(line); continue
        if data[0] == 'g':
            if conv(data[1])[1]:
                t2 = int(data[1]); s = data[2]
            else:
                t2 = 3; s = data[1]
            if t2 < t:
                short.append([t2, s]); continue
            else:
                long.append(line); continue
        invalid.append(line)
    input.close
    g = [uniq, no, long, short, invalid]
    return(g)

def writegive(giveup, g):
    try:
        output = open(giveup, 'w')
    except:
        print ('Unable to write a file:', giveup)
        return
    for i in range(3):
        for j in g[i]:
            output.write (j.strip()+'\n')
    for i in g[3]:
        output.write ('g '+str(i[0])+' '+i[1]+'\n')
    output.close
    return




# -*- coding: utf-8 -*-
import sys, os.path, copy, datetime, warnings
from .create import (append_database, show_status, analyze, reanalyze, merge, reanalyze_giveup)
from .calc import (solve, solveone, possible)
from .help import (helpmessage, advancedhelp)
from .misc import (conv, check, cell, blank, lev, duplicate, current, box, pbox)
from .output import (output, short, url, listup)

# Command interpretation

def command(arg, config):
    c = arg[0]
    if c == 'a' or c == 'ac':
        bookmark = config["bookmark"]
        if c == 'a':
            move = []
        else:
            move = config["move"]
        if len(arg) > 1:
            try:
                verbose = int(arg[1])
            except:
                verbose = 1
        else:
            verbose = 1
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        maxtime = int(config['maxtime'])
        n = pointer[level]
        n, move = show(file, move, level, n, bookmark, verbose, maxtime)
        pointer[level] = n
        return config
    if c == 'b':
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        move = config["move"]
        bookmark = config["bookmark"]
        if move == []:
            print ('Take back unavaillable because we are at initial position.')
            return config
        move = move[:len(move)-1]
        n = pointer[level]
        n, move = show (file, move, level, n, bookmark, 0, 0)
        pointer[level] = n
        config['move'] = move
        return config
    if c == 'c':
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        move = config["move"]
        bookmark = config["bookmark"]
        n = pointer[level]
        n, move = show (file, move, level, n, bookmark, 0, 0)
        if n == -1:
            print ('Going back to problem No. 1.')
            n = 1
            n, move = show (file, move, level, n, bookmark, 0, 0)
            if n == -1:
                print ('There is no problem in level {0}. Change level with l command.'.format(level))
                return config
        pointer[level] = n
        config['pointer'] = pointer
        config['move'] = move
        return config
    if c == 'h':
        print (helpmessage())
        return config
    if c == 'ha':
        print (advancedhelp())
        return config
    if c == 'i' or c == 'ii' or c == 'iii':
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        move = config["move"]
        bookmark = config["bookmark"]
        maxtime = int(config['maxtime'])
        n = pointer[level]
        type = len(c)+10
        n, move = show (file, move, level, n, bookmark, type, maxtime)
        return config
    if c == 'l':
        level = int(config["level"])
        try:
            l = int(arg[1])
        except:
            return config
        if l > 0 and l < 10:
            level = l
            config['level'] = level
        config = command(['c'], config)
        return config
    if c == 'n' or c == 'p' or c == 'j' or c == 'initial':
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        bookmark = config["bookmark"]
        n = pointer[level]
        if level == 0:
            if c == 'n' or c == 'p': return config
        else:
            n = int(n)
        if c == 'p':
            n -= 1
            if n == 0:
                print ('No previous problem.')
                n = 1
                return config
        if c == 'n': n += 1
        if c == 'j':
            if len(arg) < 2:
                return config
            try:
                n = int(arg[1])
            except:
                return config
        move = []
        n, move = show (file, move, level, n, bookmark, 0, 0)
        if n == -1:
            return config
        pointer[level] = n
        config['pointer'] = pointer
        config['move'] = move
        return config
    if c == 'u':
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        move = config["move"]
        bookmark = config["bookmark"]
        n = pointer[level]
        n, move = show (file, move, level, n, bookmark, 10, 0)
        pointer[level] = n
        return config
    if c.isdigit(): # move
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        n = pointer[level]
        move = config["move"]
        bookmark = config["bookmark"]
        row = int(c) // 100
        if row < 1 or row > 9:
            print ('Invalid move. If you want to fill Row 3 Column 5 with 7, type 357.')
            return config
        column = int(int(c) - row * 100) // 10
        num = int(c) % 10
        i = (row-1) * 9 + column-1
        move.append (int(c))
        n, move = show (file, move, level, n, bookmark, 0, 0)
        config['move'] = move
        return config
    if c == 'all':
        if len(arg) > 1:
            verbose = int(arg[1])
        else:
            verbose = 0
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        analyze(file, level, verbose)
        return config
    if c == 'append' or c == 'bp':
        file =  os.path.expanduser(config["file"])
        maxtime = int(config['maxtime'])
        bookmark = config['bookmark']
        if len(arg) > 2:
            verbose = int(arg[2])
        else:
            verbose = 1
        if len(arg) == 1:
            print ('Position not specified.')
            return(config)
        try:
            s, err = conv(arg[1])
        except:
            print ('Invalid position.')
            return config
        if err:
            print (s)
            return config
        s2 = copy.copy(s)
        s, message, level, solved, err = solve(s, 0, blank(s), maxtime)
        if solved:
            if err:
                print ('This sudoku has multiple solutions.')
            else:
                if c == 'append': # Append to book
                    infile = open(file)
                    for line in infile:
                        data = line.strip().split(' ')
                        s3 = conv(data[1])[0]
                        if s2 == s3:
                            print ('This sudoku is already in the book.')
                            infile.close
                            return config
                    infile.close
                    out = openappend(file)
                    if output == 'error':
                         print ('Unable to write a file:', file)
                         return config
                    out.write (str(level)+' '+short(s2)+'\n')
                    out.close
                    print ('Valid sudoku. Appended to the book by level {0}.'.format(level))
                else: # Add to bookmark
                    problem = short(s2)
                    comment = ''
                    if 'bookmark' in config:
                        bookmark = config['bookmark']
                        num = 1
                        while 'b'+str(num) in bookmark:
                            num += 1
                        label = 'b'+str(num)
                        for lab in bookmark:
                            if bookmark[lab]['problem'] == problem and bookmark[lab]['move'] == mo:
                                label = lab
                                print ('Already labeled as {0}'.format(label))
                                if 'comment' in bookmark[label]:
                                    comment = bookmark[label]['comment']
                                    print ('Comment = {0}'.format(comment))
                                    break
                                else:
                                    bookmark = {}
                    try:
                        com = input('Comment on this position: ')
                    except:
                        com = ''
                        print ('')
                    if com != '': comment = com
                    bookmark[label] = {}
                    bookmark[label]['problem'] = problem
                    bookmark[label]['move'] = ''
                    bookmark[label]['added'] = datetime.datetime.now().strftime('%Y/%m/%d')
                    if comment != '':
                        bookmark[label]['comment'] = comment
                    config['bookmark'] = bookmark
        else:
            if err:
                print ('This sudoku has no solution.')
            else:
                print ('This sudoku cannot be solved in {0} seconds.'.format(maxtime))
        return config
    if c == 'book':
        file =  os.path.expanduser(config["file"])
        show_status(file)
        return config
    if c == "config":
        for i in config:
            if i != 'bookmark':
                print ('{0} = {1}'.format(i,config[i]))
        return config
    if c == 'create':
        if len(arg) > 1:
            n = int(arg[1])
        else:
            n = 10
        file =  os.path.expanduser(config["file"])
        giveup =  os.path.expanduser(config["giveup"])
        print ('Creating {0} new problems.'.format(n))
        append_database(file, giveup, n)
        return config
    if c == 'ba': # add bookmark
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        move = config["move"]
        bookmark = config["bookmark"]
        n = pointer[level]
        if level == 0: # bookmark
            if n not in bookmark:
                print ('Label {0} not found in bookmark.')
                print ('Going to level 3.')
                config['level'] = 3
                return config
            if 'problem' not in bookmark[n]:
                print ('Label {0} broken.')
                print ('Going to level 3.')
                config['level'] = 3
                return config
            problem = bookmark[n]['problem']
        else:
            n = int(n)
            infile = open(file, 'r')
            no = 0
            problem = ''
            for line in infile:
                data = line.strip().split(' ')
                if int(data[0]) == level:
                    no += 1
                    if no == n:
                        infile.close
                        problem = data[1]
                        break
            if problem == '':
                pointer[level] = 1
                print ('Level {0} no. {1} not found. Going back to no. 1'.format(level, n))
                return config
        s, err = conv(problem)
        if err:
            print (s)
            print ('This problem is not valid.')
            return config
        s2 = copy.copy(s)
        if len(move) > 0:
            s2, move, message, err = current(s2, move)
            if err:
                print (message)
                return config
        problem = short(s)
        mo = ''
        for i in move:
            mo += str(i)
        if len(arg) > 1:
            label = arg[1]
        else:
            label = 'b1'
        comment = ''
        if 'bookmark' in config:
            bookmark = config['bookmark']
            num = 1
            for i in bookmark:
                num += 1
            label = 'b'+str(num)
            for lab in bookmark:
                if bookmark[lab]['problem'] == problem and bookmark[lab]['move'] == mo:
                    label = lab
                    print ('Already labeled as {0}'.format(label))
                    if 'comment' in bookmark[label]:
                        comment = bookmark[label]['comment']
                        print ('Comment = {0}'.format(comment))
                    break
        else:
            bookmark = {}
        try:
            com = input('Comment on this position: ')
        except:
            com = ''
            print ('')
        if com != '': comment = com
        bookmark[label] = {}
        bookmark[label]['problem'] = problem
        bookmark[label]['move'] = mo
        bookmark[label]['added'] = datetime.datetime.now().strftime('%Y/%m/%d')
        if comment != '':
            bookmark[label]['comment'] = comment
        config['bookmark'] = bookmark
        return config
    if c == 'bl':
        if 'bookmark' in config:
            bookmark = config['bookmark']
        else:
            print ('No bookmark.')
            return config
        for label in bookmark:
            if 'comment' in bookmark[label]:
                comment = bookmark[label]['comment']
            else:
                comment = ''
            if 'added' in bookmark[label]:
                added = bookmark[label]['added']
            else:
                added = ' ' * 10
            print ('{0}   {1}   {2}'.format(added, label, comment))
        return config
    if c == 'br':
        file =  os.path.expanduser(config["file"])
        if 'bookmark' in config:
            bookmark = config['bookmark']
        else:
            print ('No bookmark.')
            return config
        if len(arg) == 1:
            print ('Label is not specified.')
            return config
        n = arg[1]
        level = 0
        if n not in bookmark:
            print ('Label {0} not found in bookmark.'.format(n))
            print ('Going to level 3.')
            config['level'] = 3
            return config
        mo = bookmark[n]['move']
        move = []
        while len(mo) > 2:
            move.append (int(mo[:3]))
            mo = mo[3:]
        n, move = show (file, move, level, n, bookmark, 0, 0)
        config['level'] = level
        config['move'] = move
        pointer = config['pointer']
        pointer[level] = n
        return config
                
    if c == 'giveup':
        if len(arg) > 1:
            t = int(arg[1])
        else:
            t = 10
        giveup =  os.path.expanduser(config["giveup"])
        reanalyze_giveup(giveup, t)
        return config        
    if c == 'import':
        file =  os.path.expanduser(config["file"])
        file2 =  os.path.expanduser(config["file2"])
        merge(file, file2)
        return config
    if c == 'reanalyze':
        file =  os.path.expanduser(config["file"])
        file2 =  os.path.expanduser(config["file2"])
        reanalyze(file, file2)
        return config
    if c == 'solve':
        maxtime = int(config['maxtime'])
        if len(arg) > 2:
            verbose = int(arg[2])
        else:
            verbose = 1
        if len(arg) == 1:
            print ('Position not specified.')
            return(config)
        try:
            s, err = conv(arg[1])
        except:
            print ('Invalid position.')
            return config
        if err:
            print (s)
            return config
        print (output(s))
        message, err = check(s)
        if err:
            print (message)
            return config
        solveprint(s, verbose, blank(s), maxtime)
        return config
    if c == 'sp':
        file =  os.path.expanduser(config["file"])
        level = int(config["level"])
        pointer = config['pointer']
        move = config["move"]
        bookmark = config["bookmark"]
        maxtime = int(config['maxtime'])
        n = pointer[level]
        type = 20
        n, move = show(file, move, level, n, bookmark, type, maxtime)
        config['move'] = move
        pointer[level] = n
        return config
    print ('Invalid command. Type h for help.')
    return config

# Show current position (from a, s, n, p, i, u commands)

def show(file, move, level, n, bookmark, type, maxtime):
    infile = open(file, 'r')
    no = 0
    if level == 0: # bookmark
        if n not in bookmark:
            print ('Label {0} not found in bookmark.')
            print ('Going to level 3.')
            config['level'] = 3
            return config
        if 'problem' not in bookmark[n]:
            print ('Label {0} broken.')
            print ('Going to level 3.')
            config['level'] = 3
            return config
        problem = bookmark[n]['problem']
    else:
        n = int(n)
        infile = open(file, 'r')
        no = 0
        problem = ''
        for line in infile:
            data = line.strip().split(' ')
            if int(data[0]) == level:
                no += 1
                if no == n:
                    infile.close
                    problem = data[1]
                    break
        infile.close
        if problem == '':
            print ('Level {0} no. {1} not found.'.format(level, n))
            return -1, move # not found

    s, err = conv(problem)

    if err:
        print (s)
        print ('This problem is not valid. Going to next problem.')
        return n+1, move
    if len(move) > 0:
        s, move, message, err = current(s, move)
        if err:
            print (message)
        status = 'move '+str(len(move))
    else:
        status = 'initial position'
    if blank(s) == 0:
        status = 'Final position'
    if type == 0: # show
        if level == 0:
            if 'comment' in bookmark[n]:
                comment = bookmark[n]['comment']
            else:
                comment = ''
            print ('Bookmark {0} {1} : {2}'.format(n, comment, status))
        else:
            print ('Level {0} No. {1}: {2}'.format(level, n, status))
        print (output(s))
        if blank(s) == 0:
            print ('Now this problem is solved !')
        else:
            print ('Type 3 digits (row, column, number) to put a number. i for hint.')
    if type > 0 and type < 6:
        print ('\nLevel {0} No. {1}: {2}'.format(level, n, status))
        solveprint(s, type, blank(s), maxtime)                   
    if type == 10: # url
        print (url(s))
    if type == 11 or type == 12 or type == 13 or type == 20: # prepare solving
        if blank(s) == 0:
            print ('Already solved.')
            return n, move
        s2 = copy.copy(s)
        s2, message, level, solved, err = solve(s2, 0, blank(s), maxtime)
        if err:
            if solved:
                print ('This position has multiple solutions.')
            else:
                print ('There is no solution for this position. You can take back one move with b.')
            return n, move
        p = possible(s)
        b = box()
        pb = pbox()
        blank1 = blank(s)
        start = datetime.datetime.now()
        endtime = start + datetime.timedelta(seconds=maxtime)
    if type == 11 or type == 12 or type == 13: # show hint
        s2, p, message, logic, depth, found, err = solveone(s,p, 4, 0, blank1, endtime, b, pb)
        if logic == 'Naked single' or logic == 'Hidden single':
            if logic == 'Naked single':
                print ('Look at {0}. What number is available?'.format(message[14:18]))
            else:
                print (message[:message.index(':')]+'can be found.')
        else:
            if type == 11:
                print ('Think candidates of the cells.\nYou can manipulate candidates of this position at')
                print (url(s))
                print ('For more hints, type ii.')
            if type == 12 or type == 13:
                logi= [logic]; mes = [message]
                while blank(s2) == blank1:
                    s, p, message, logic, depth, found, err = solveone(s,p, 4, 0, blank1, endtime, b, pb)
                    logi.append(logic)
                    mes.append(message)
                if type == 12:
                    if len(logi) > 1: print ('Following logics are successively used.')
                    for i in range(len(logi)):
                        print (logi[i])
                    print ('See full explanation by typing iii.')
                else:
                    for message in mes: print (message)
    if type == 20: # Solve partially
        logic = 'Naked single'
        while (logic == 'Naked single' or logic == 'Hidden single') and blank(s) > 0:
            s2 = copy.copy(s)
            s, p, message, logic, depth, found, err = solveone(s,p, 4, 0, blank(s), endtime, b, pb)
            if logic == 'Naked single' or logic == 'Hidden single': print (message)
            for i in range(81):
                if s[i] != s2[i]:
                    j = (i // 9) * 10 + i % 9 + 11
                    m = j * 10 + s[i]
                    move.append (m)
        print ('\n' + output(s))
    return n, move

# Analyze a problem (from a command through show)

def solveprint(s, verbose, maxdepth, maxtime):

    s, message, level, solved, err = solve(s, verbose, maxdepth, maxtime)
    if err:
        if verbose > 0: print (message)
        if solved:
            if verbose > 0: print ("Invalid sudoku with multiple solutions.")
            if verbose > 4:
                print ('Solution 1.')
                print (output (s[0]))
                print ('Solution 2.')
                print (output (s[1]))
                s = duplicate(s[0],s[1])
                print ('Common numbers in the 2 solutions')
                print (output (s))
        else:
            if verbose > 0: print ("Invalid sudoku with no solution.")
        sys.exit()
    if solved:
        if verbose > 0:
            print ('Valid sudoku with unique solution of level {0} ({1}).'.format(level,lev(level)))
        if verbose == 4:
            print ('Here is the solution.')
            print (output (s))
    else:
        print ('\nGive up with {0} blank cells.'.format(blank(s)))
        print (output (s))
    
    return



# -*- coding: utf-8 -*-
import copy, datetime, itertools
from .misc import (box, pbox, boxlist, combmir, pairs, check, cell, blank)
from .output import (output, short)
from .wing import (xwing, xywing)
from .chain import (pairchain)
from .search import (trial, search)

def solve(s, verbose, maxdepth, maxtime):
    solved = False
    p = possible(s)
    b = box()
    pb = pbox()
    search = []
    start = datetime.datetime.now()
    endtime = start + datetime.timedelta(seconds=maxtime)
    LevelPoint = int((blank(s)/15.0)**3) # 30 -> 8, 64 -> 77
    while (not solved):
        bl = blank(s)
        s, p, message, logic, depth, found, err = solveone(s,p, verbose, 0, maxdepth, endtime, b, pb)
        if err:
            if found:
                return (s, message, 0, True, True)
            else:
                return (s, message, 0, False, True)
        if found:
            if verbose == 4: message = str(81 - bl) + ': ' + message
            if message != '': print (message)
            if verbose > 4 and blank(s) < bl:
                print (str(81 - blank(s))+ ' numbers determined')
                print (output(s))
            LevelPoint += {
              'Naked single': 0,
              'Hidden single':  3,
              'Pointing pair': 25,
              'Pointing triple': 25,
              'Naked pair': 25,
              'Naked triple': 40,
              'Hidden pair': 50,
              'Hidden triple': 70,
              'X-wing': 70,
              'XY-wing': 70,
              'XYZ-wing': 100, # not implemented yet
              'Swordfish': 100, # not implemented, rare 
              'Jellyfish': 100, # not implemented, rare
              "Bowman's Bingo": 100, # not sure
              'Naked quad': 100, # not implemented yet
              'Hidden quad': 100, # not implemented yet
              'Remote pairs': 100, # not implemented yet http://www.sudokuwiki.org/Remote_Pairs
              'Chain of pairs': 200,
              'Chain of pairs (long)': 300,
              'Trial': 500,
              'Trial (deep)': 1000,
              'Search': 1500,
              'Deep search': 2500
            }[logic]
        else:
            return (s, 'Give up', 0, False, False)
        if blank(s) == 0:
            solved = True
        else:
            solved = False
    for i in range(9):
        if [0, 8, 45, 70, 120, 320, 1000, 2000, 5500][i] <= LevelPoint:
            level = i+1
    return (s, 'Solved', level, True, False)

############################################
#                               Solve one move
############################################

def solveone(s, p, verbose, depth, maxdepth, endtime, b, pb):

    # Check validity
    message, err = check(s)
    if verbose == 0: message = ''
    if err: return (s, p, message, 0, depth, False, True)

    # Basic logic
    boxl = boxlist(s)
    s, p, message, found, err = naksing(s, p, b, verbose) # Naked single
    if found or err: return (s, p, message, 'Naked single', depth, found, err)

    # Intermediate logic
    comb, mirror = combmir(p, boxl)
    if depth < 2:
        s, p, message, found, err = hidsing(s, p, verbose) # Hidden single
        if found or err: return (s, p, message, 'Hidden single', depth, found, err)
        s, p, logic, message, found, err = pointing(s, p, pb, verbose) # Pointing pair or triple
        if found or err: return (s, p, message, logic, depth, found, err)
        if depth < 1:
            onlypair = False
        else:
            onlypair = True
        s, p, logic, message, found, err = nakhid(s, p, boxl, comb, mirror, verbose, onlypair) # Naked or hidden
        if found or err: return (s, p, message, logic, depth, found, err)
    
    # Advanced logic
    pair, paircomb = pairs(s,p)
    if depth == 0:
        s, p, message, found, err = xwing(s, p, b, boxl, mirror, verbose) # X-wing
        if found or err: return (s, p, message, 'X-wing', depth, found, err)
        s, p, message, found, err = xywing(s, p, b, pair, verbose) # XY-wing
        if found or err: return (s, p, message, 'XY-wing', depth, found, err)

        if len(pair) > 3:
            s, p, message, chainlength, found, err = pairchain(s, p, b, pair, paircomb, verbose) # Chain of pairs
            if chainlength < 8:
                logic = 'Chain of pairs' # Short chain
            else:
                logic = 'Chain of pairs (long)' # Long chain
            if found or err: return (s, p, message, logic, depth, found, err)

    # Trial
    maxstep = 15 # Maximum steps of scan distinguished from full search of 1 depth
    if depth == 0 and maxdepth < 999:
        s, p, step, message, found, err = trial(s, p, maxstep, verbose, b, pair)
        if step < 10:
            logic = 'Trial'
        else:
            logic = 'Trial (deep)'
        if found or err: return (s, p, message, logic, depth, found, err)    

    # Search from minimum depth
    if maxdepth == 999:
        bla = blank(s) # Creating mode
    else:
        bla = 0
       
    if depth == 0 :
        if maxdepth > 2 and bla < 60:
            s, p, message, logic, depth2, found, err = search(s, p, verbose, 1, 2, endtime, b, pb)
            if found or err: return (s, p, message, logic, depth2, found, err)    
        if maxdepth > 3 and bla < 45:
            s, p, message, logic, depth2, found, err = search(s, p, verbose, 1, 3, endtime, b, pb)
            if found or err: return (s, p, message, logic, depth2, found, err)    
        if maxdepth > 4 and bla < 35:
            s, p, message, logic, depth2, found, err = search(s, p, verbose, 1, 4, endtime, b, pb)
            if found or err: return (s, p, message, logic, depth2, found, err)    

    # Full search
    if maxdepth > depth:
        s, p, message, logic, depth, found, err = search(s, p, verbose, depth+1, maxdepth, endtime, b, pb)
        if found or err: return (s, p, message, logic, depth, found, err)

    return (s, p, '', 0, depth, False, False)

def possible(s):
    p = [[]] * 81
    b = box()
    for i in range(81):
        if s[i] == 0:
            p[i] = [1] * 9
            for i2 in b[i]:
                if s[i2] > 0: p[i][s[i2]-1] = 0
        else:
            p[i] = [0] * 9
            p[i][s[i]-1] = 1
    return (p)

# Naked single

def naksing(s, p, b, verbose):
    
    for i in range(81):
        if s[i] == 0:
            for i2 in b[i]:
                v = s[i2]
                if v > 0:
                    p[i][v-1] = 0
        else:
            p[i] = [10]

    found = False; message = ''
    for i in range(81):
        if (sum(p[i])) == 0:
            if verbose > 0: message = 'This sudoku has no solution because ' + cell(i) + ' has no candidate.'
            return (s, p, message, False, True)
        if (sum(p[i])) == 1:
             i2 = p[i].index(1)
             found = True
             s[i] = i2+1
             if verbose > 3:
                 if message == '':
                      message = ('Naked single: ')
                 else:
                      message = message + ", "
                 message = message + cell(i) + " = " + str(i2+1)
                 
    if blank(s) == 0:
        message2, err = check(s)
        if verbose == 0: message2 = ''
        if err: return (s, p, message2, False, True)

    return (s, p, message, found, False)

# Hidden single

def hidsing(s, p, verbose):

    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        for n in range(9):
            sum = 0
            for i in (0,1,2,9,10,11,18,19,20):
                if s[begin + i] == 0:
                    sum = sum + p[begin + i][n]
            if sum == 1:
                for i in (0,1,2,9,10,11,18,19,20):
                    if (s[begin + i] == 0) and (p[begin + i][n]) == 1:
                        s[begin + i] = n+1
                        if verbose < 4:
                            message = ''
                        else:
                            message = 'Hidden single in box ' + str(b+1) + ' : ' + cell(begin + i) + ' = ' + str(n+1)
                        return (s, p, message, True, False)

    for i in range(9):
        for n in range(9):
            sum = 0
            for j in range(9):
                if s[i*9+j] == 0:
                    sum = sum + p[i*9+j][n]
            if sum == 1:
                for j in range(9):
                    if (s[i*9+j] == 0) and (p[i*9+j][n]) == 1:
                        s[i*9+j] = n+1
                        if verbose < 4:
                            message = ''
                        else:
                            message = 'Hidden single in row ' + str(i+1) + ' : ' + cell(i*9+j) + ' = ' + str(n+1)
                        return (s, p, message, True, False)

    for j in range(9):
        for n in range(9):
            sum = 0
            for i in range(9):
                if s[i*9+j] == 0:
                    sum = sum + p[i*9+j][n]
            if sum == 1:
                for i in range(9):
                    if (s[i*9+j] == 0) and (p[i*9+j][n]) == 1:
                        s[i*9+j] = n+1
                        if verbose < 4:
                            message = ''
                        else:
                            message = 'Hidden single in column ' + str(j+1) + ' : ' + cell(i*9+j) + ' = ' + str(n+1)
                        return (s, p, message, True, False)

    return (s, p, '', False, False)

# Pointing pair or triple

def pointing(s, p, pb, verbose):

    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        blank = set()
        for i in range(9):
            if s[begin+(0,1,2,9,10,11,18,19,20)[i]] == 0:
                blank.add(i)
        for i in range(6):
            j = [set([0,1,2]),set([3,4,5]),set([6,7,8]),set([0,3,6]),set([1,4,7]),set([2,5,8])][i]
            b2 = blank & j
            if len(b2) > 1 > 0:
               for n in range(9):
                   sum = 0
                   for a in b2: sum = sum + p[begin + (0,1,2,9,10,11,18,19,20)[a]][n]
                   if sum > 1:
                       sum2 = 0; sum3 = 0
                       for a in (blank - b2): sum2 = sum2 + p[begin + (0,1,2,9,10,11,18,19,20)[a]][n]
                       for a in pb[(b,i)]:
                           if s[a] == 0: sum3 = sum3 + p[a][n]
                       if (sum2 == 0) != (sum3 ==0):
                           logic = 'Pointing '+ ['','','pair', 'triple'][sum]
                           message = ''
                           if verbose > 2: message = logic + ' in box '+str(b+1)
                           if verbose > 3: message = message + " removed "+str(n)+" from "
                           if (sum2 ==0):
                                for a in pb [(b,i)]:
                                   if s[a] == 0 and p[a][n] == 1:
                                       p[a][n]=0
                                       if verbose > 3: message = message + cell(a) + ' '
                                       if p[a].count(1) == 1:
                                           if verbose > 3: message = message + '(='+str(p[a].index(1)+1) + ') '
                                           s[a] = p[a].index(1)+1
                           else:
                               for a in (blank - b2):
                                   a2 = begin + (0,1,2,9,10,11,18,19,20)[a]
                                   if p[a2][n] == 1:
                                       r=p[a2]; r[n]=0; p[a2]=r
                                       if verbose > 3: message = message + cell(a2) + ' '
                                       if p[a2].count(1) == 1:
                                           if verbose > 3: message = message + '(='+str(p[a2].index(1)+1) + ') '
                                           s[a2] = p[a2].index(1)+1
                           return (s, p, logic, message, True, False)

    return (s, p, '', '', False, False)

###########################
# Naked and hidden pair or triple
###########################

def nakhid(s, p, boxl, comb, mirror, verbose, onlypair):

    # Naked pair
    for i in range(len(comb)):
        list = comb[i]
        if len(list) > 3:
            np, pa, position, f = nakpair(list)
            if np:
                message = ''
                if verbose > 2: message =  'Naked pair in ' + ['box ', 'row ', 'column '][i // 9] + str(i % 9+1)
                if verbose > 3: message = message + ' made removal from'
                for j in f:
                    k = boxl[i][j]
                    p[k][pa[0]-1] = 0
                    p[k][pa[1]-1] = 0
                    if verbose > 3: message = message + ' ' + cell(k)
                    if p[k].count(1) == 1:
                        if verbose > 3: message = message + '(='+str(p[k].index(1)+1) + ') '
                        s[k] = p[k].index(1)+1
                return (s, p, 'Naked pair', message, True, False)
 
    # Naked triple
    if not onlypair: 
	    for i in range(len(comb)):
	        list = comb[i]
	        if len(list) > 5:
	            nt, tri, position, f = naktriple(list)
	            if nt:
	                message = ''
	                if verbose > 2: message =  'Naked triple in ' + ['box ', 'row ', 'column '][i // 9] + str(i % 9+1)
	                if verbose > 3: message = message + ' made removal from'
	                for j in f:
	                    k = boxl[i][j]
	                    p[k][tri[0]-1] = 0
	                    p[k][tri[1]-1] = 0
	                    p[k][tri[2]-1] = 0
	                    if verbose > 3: message = message + ' ' + cell(k)
	                    if p[k].count(1) == 1:
	                        if verbose > 3: message = message + '(='+str(p[k].index(1)+1) + ') '
	                        s[k] = p[k].index(1)+1
	                return (s, p, 'Naked triple', message, True, False)
 
    # Hidden pair
    for i in range(len(mirror)):
        list2 = mirror[i][1]
        if len(list2) > 3:
            hp, pa, position, f = nakpair(list2)
            if hp:
                p1 = mirror[i][0][position[0]]; p2 = mirror[i][0][position[1]]
                message = 'Hidden pair of (' + str(p1) +', ' + str(p2) + ') in ' + ['box ', 'row ', 'column '][i // 9] + str(i % 9+1) + ': ' + cell(boxl[i][pa[0]]) + ' ' + cell(boxl[i][pa[1]])
                r = [0] * 9; r[p1-1] = 1; r[p2-1] = 1
                p[boxl[i][pa[0]]] = r; p[boxl[i][pa[1]]] = copy.copy(r)
                message = ''
                if verbose > 2: message =  'Hidden pair in ' + ['box ', 'row ', 'column '][i // 9] + str(i % 9+1)
                return (s, p, 'Hidden pair', message, True, False)

     # Hidden triple
    if not onlypair: 
	    for i in range(len(mirror)):
	        list2 = mirror[i][1]
	        if len(list2) > 3:
	            ht, tri, position, f = naktriple(list2)
	            if ht:
	                rem = set([1,2,3,4,5,6,7,8,9])
	                for k in range(3):
	                    rem.remove (mirror[i][0][position[k]])
	                message = 'Hidden triple of ' + str(tri) + ' in ' + ['box ', 'row ', 'column '][i // 9] + str(i % 9+1) + ': ' + cell(boxl[i][tri[0]]) + ' ' + cell(boxl[i][tri[1]]) + ' ' + cell(boxl[i][tri[2]])
	                for k in range(3):
	                    for m in rem:
	                        p[boxl[i][tri[k]]][m-1] = 0
	
	                message = ''
	                if verbose > 2: message =  'Hidden triple in ' + ['box ', 'row ', 'column '][i // 9] + str(i % 9+1)
	                return (s, p, 'Hidden triple', message, True, False)

    return (s, p, 5, '', False, False)

def nakpair(list):
    pair = []
    for i in list:
        if len(i) == 2: pair.append (i)
    if len(set(pair)) < len(pair):
        pairs = []
        for i in range(len(pair)):
            for j in range(i+1,len(pair)):
                if pair[i] == pair[j]: pairs.append (pair[i])
        for pa in pairs:
            other = set(list) - set([pa])
            f = []
            for q in other:
                if pa[0] in q or pa[1] in q:
                    f.append (list.index(q))
            if len(f) > 0:
                position = [i for i, x in enumerate(list) if x == pa]
                return (True, pa, position, f)
    
    return (False, 0, 0, 0)

def naktriple(list):
    triple = []
    num = set()
    for i in list:
        if len(i) < 4:
            triple.append (i)
            for j in i: num.add (j)
        for c in itertools.combinations(num,3):
            n = 0
            for k in triple:
                if set(k).issubset(set(c)): n += 1
            if n == 3:
                tri = []; position = []
                m = 0; f = []
                for k in list:
                    if set(k).issubset(set(c)):
                        tri.append(k)
                        position.append(m)
                    else:
                        if set(k).intersection(set(c)):
                            f.append(m)
                    m += 1
                if len(f) > 0:
                     return (True, c, position, f)
    
    return (False, 0, 0, 0)




#  Chain of pairs

def pairchain(s, p, b, pair, paircomb, verbose):
    from .misc import (cell)
    message = ''
    if len(pair) < 4: return (s, p, message, False, False)
    for i in paircomb:
        chain = [i]
        chaincell = [i[0]]
        chain2 = findchain(pair, i, b)
        key = i[0]*10+i[1]
        dict = {key:chain2}
        path = {key:[i]}
        if chain2 != []:
            for j in chain2:
                chaincell.append (j[0])
            for j in chain2:
                chain.append(j)
                path[j[0]*10+j[1]] = path[i[0]*10+i[1]] + [j]
            found = True; loop = False
            while found:
                found = False
                for j in chain2:
                    jkey = j[0]*10 + j[1]
                    if jkey in dict:
                        chain3 = dict[jkey]
                    else:
                        chain3 = findchain(pair, j, b)
                        dict[jkey] = chain3
                    for j3 in chain3:
                        path[j3[0]*10+j3[1]] = path[jkey] + [j3]
                    if chain3 != []:
                        for j2 in chain3:
                            if j2[0] in chaincell:
                                for k in chain:
                                    if k[0] == j2[0]:
                                        if k[1] == j2[1]:
                                            loop = True
                                        else:
                                            # Found a chain
                                            path1 = path[j2[0]*10+j2[1]]
                                            path2 = path[j2[0]*10+k[1]]
                                            for i in path1[1:]:
                                                if i in path2:
                                                    p1 = path1.index(i)
                                                    path1 = path1[p1:]
                                                    p2 = path2.index(i)
                                                    path2 = path2[p2:]
                                            i = path1[0]
                                            p[i[0]][i[1]-1] = 0
                                            n = p[i[0]].index(1) + 1
                                            s[i[0]] = n
                                            chainlength = len(path1) + len(path2)
                                            if verbose > 1:
                                                if verbose == 2:
                                                    message = 'Chain of pairs. Start from '+cell(i[0]) + '.'
                                                else:
                                                    message = 'Chain of pairs. Assume that '+cell(i[0])+' is '+str(i[1])+' and we have following chains.\n'
                                                    message += '(1) ' + chainpath(path1) + '\n' + '(2) ' + chainpath(path2) + '\n'
                                                    message += 'Now we have contradiction on '+cell(j2[0])+'. Therefore '+cell(i[0])+' should be '+str(n)+'. '
                                            return (s, p, message, chainlength, True, False)
                            chaincell.append (j2[0])
                        if not loop:
                            chain2 = chain3
                            for j in chain2: chain.append(j)
                            found = True

    return (s, p, message, 0, False, False)

def findchain(pair, i, b):
    chain = []
    for j in pair:
        if j[0] in b[i[0]] and (i[1] == j[1] or i[1] == j[2]):
            if i[1] == j[1]:
                chain.append ([j[0], j[2]])
            else:
                chain.append ([j[0], j[1]])
    return chain

def chainpath(path):
    from .misc import (cell)
    chainpath = ''
    for i in path:
        if i != path[0]: chainpath += ' >> '
        chainpath += cell(i[0]) + ' = ' + str(i[1])
    return chainpath



# -*- coding: utf-8 -*-
from .misc import (conv, blank)
def output(s):
    
    out = '  1 2 3 4 5 6 7 8 9\n'
    out += ' +-----+-----+-----+\n'
    for i in range(9):
        out += str(i+1) + '|'
        for j in range(9):
            v = s[i*9+j]
            if v == 0:
                out += ' '
            else:
                out = out + str(v)
            if j % 3 == 2:
                out += "|"
            else:
            	out += ' '
        out += '\n'
        if i % 3 == 2:
            out += ' +-----+-----+-----+\n'
    return out

def url(s):
    out = "http://www.sudoku-solutions.com/index.php?puzzle="
    for i in range(81):
        v = str(s[i])
        if v == 0:
            out = out + '.'
        else:
            out = out + str(v)
    return out

def short(s):
    sudoku = ''
    for j in s:
        sudoku += str(j)
    return sudoku

# not using now

def listup(file, level):
    input = open(file, 'r')
    i = 0
    for line in input:
        data = line.strip().split(' ')
        s = conv(data[1])[0]
        i += 1
        if int(data[0]) == level:
            print ('No.{0} with {1} numbers starting from {2}'.format(i,81-blank(s),data[1][:18]))
    input.close
    return
